package authutil

import (
	"context"
	"fmt"
	"github.com/go-micro/plugins/v4/auth/jwt"
	"github.com/go-redsync/redsync/v4"
	"github.com/go-redsync/redsync/v4/redis/goredis/v9"
	"github.com/google/uuid"
	goredislib "github.com/redis/go-redis/v9"
	"github.com/sirupsen/logrus"
	"go-micro.dev/v4/auth"
	"os"
	"os/signal"
	"sghgogs.com/sghblog/common"
	"sghgogs.com/sghblog/common/errorcode"
	"strconv"
	"strings"
	"sync"
	"syscall"
	"time"
)

var (
	JWTAuthService     *JWTAuth
	jwtAuthServiceOnce sync.Once
)

const (
	expiry = time.Second * time.Duration(24*3600)
	// expiry = time.Second * time.Duration(1*3600)
)

type JWTAuth struct {
	Mu        *redsync.Redsync
	Client    *goredislib.Client
	Auth      auth.Auth
	namespace string
	Enable    bool
}

func NewJWTAuth(client *goredislib.Client, namespace string, enable bool) *JWTAuth {
	pubKey, err := os.ReadFile("key/sample_key.pub")
	if err != nil {
		logrus.Fatalf("Unable to read public key: %v", err)
	}
	privKey, err := os.ReadFile("key/sample_key")
	if err != nil {
		logrus.Fatalf("Unable to read private key: %v", err)
	}

	jwtAuthServiceOnce.Do(func() {
		newAuth := jwt.NewAuth(
			auth.Namespace(namespace),
			auth.PrivateKey(string(privKey)),
			auth.PublicKey(string(pubKey)),
		)

		pool := goredis.NewPool(client) // or, pool := redigo.NewPool(...)

		// Create an instance of redisync to be used to obtain a mutual exclusion
		// lock.
		rs := redsync.New(pool)
		JWTAuthService = &JWTAuth{
			Mu:        rs,
			Client:    client,
			Auth:      newAuth,
			namespace: namespace,
			Enable:    enable,
		}
		handleSignals(client)
	})

	return JWTAuthService
}

// GenerateToken 示例：生成令牌
func (svc *JWTAuth) GenerateToken(userID int64, provider, withType, secret string, scopes []string, md map[string]string) (*auth.Account, error) {
	var account *auth.Account
	lock := svc.Mu.NewMutex(fmt.Sprintf("generate-token-lock-%d", userID))
	// 获取锁，保证原子性
	if err := lock.Lock(); err != nil {
		return account, err
	}
	defer lock.Unlock()
	// 很重要
	md["expiry"] = fmt.Sprintf("%d", time.Now().Add(expiry).Unix())
	generate, err := svc.Auth.Generate(
		fmt.Sprintf("%d", userID),
		auth.WithType(withType),
		auth.WithProvider(provider),
		auth.WithScopes(strings.Join(scopes, ",")),
		auth.WithSecret(secret),
		auth.WithMetadata(md),
	)
	if err != nil {
		code := common.FailedToGenerateTokenErrorCode
		return account, errorcode.New(svc.namespace, common.ErrorMessage[code], int32(code))
	}
	return generate, nil
}

func (svc *JWTAuth) RefreshToken(token string) (*auth.Token, error) {
	var authToken *auth.Token
	lock := svc.Mu.NewMutex("refresh-generate-token-lock")
	// 获取锁，保证原子性
	if err := lock.Lock(); err != nil {
		return authToken, err
	}
	defer lock.Unlock()
	inspect, err := svc.Auth.Inspect(token)
	if err != nil {
		return authToken, err
	}
	inspect.Metadata["expiry"] = fmt.Sprintf("%d", time.Now().Add(expiry).Unix())
	generate, err := svc.Auth.Generate(
		inspect.Metadata["id"],
		auth.WithType("user"),
		auth.WithProvider("system"),
		auth.WithScopes(strings.Join(inspect.Scopes, ",")),
		auth.WithSecret(inspect.Metadata["password"]),
		auth.WithMetadata(inspect.Metadata),
	)
	if err != nil {
		code := common.FailedToGenerateTokenErrorCode
		return authToken, errorcode.New(svc.namespace, common.ErrorMessage[code], int32(code))
	}
	userId, _ := strconv.ParseInt(inspect.Metadata["id"], 10, 64)
	return svc.Token(userId, generate.Secret)
}

// Token 重新刷新token
func (svc *JWTAuth) Token(userID int64, accessToken string) (*auth.Token, error) {
	var authToken *auth.Token
	lock := svc.Mu.NewMutex(fmt.Sprintf("token-lock-%d", userID))
	// 获取锁，保证原子性
	if err := lock.Lock(); err != nil {
		return authToken, err
	}
	defer lock.Unlock()
	return svc.Auth.Token(
		auth.WithExpiry(expiry),
		auth.WithCredentials(fmt.Sprintf("%d", userID), accessToken),
		auth.WithToken(accessToken),
	)
}

// Inspect 检测token 有效期
func (svc *JWTAuth) Inspect(accessToken string) (*auth.Account, error) {
	var account *auth.Account
	lock := svc.Mu.NewMutex(fmt.Sprintf("inspect-lock-%d", uuid.New()))
	// 获取锁，保证原子性
	if err := lock.Lock(); err != nil {
		return account, err
	}
	defer lock.Unlock()

	return svc.Auth.Inspect(accessToken)
}

// StoreToken 存储令牌到Redis
func (svc *JWTAuth) StoreToken(ctx context.Context, ID int64, name, accessToken string) error {
	lock := svc.Mu.NewMutex(fmt.Sprintf("store-token-lock-%d", ID))
	// 获取锁
	if err := lock.Lock(); err != nil {
		return err
	}

	access := fmt.Sprintf("auth:access-token:%v:%s", ID, name)
	err := svc.Client.Set(ctx, access, accessToken, expiry).Err()
	if err != nil {
		err = errorcode.New(svc.namespace, common.ErrorMessage[common.FailedToStoreTokenErrorCode], int32(common.FailedToStoreTokenErrorCode))
		logrus.Error(err)
		return err
	}
	return nil
}

// StoreRevoke 从Redis中删除令牌
func (svc *JWTAuth) StoreRevoke(ctx context.Context, ID int64, name string) error {
	lock := svc.Mu.NewMutex(fmt.Sprintf("store-revoke-lock-%d", ID))
	// 获取锁
	if err := lock.Lock(); err != nil {
		return err
	}
	defer lock.Unlock()
	access := fmt.Sprintf("auth:access-token:%v:%s", ID, name)
	err := svc.Client.Del(ctx, access).Err()
	if err != nil {
		err = errorcode.New(svc.namespace, common.ErrorMessage[common.TokenDeletionFailedErrorCode], int32(common.TokenDeletionFailedErrorCode))
		logrus.Error(err)
		return err
	}
	return nil
}

// StoreVerify 检查是否在有效期内
func (svc *JWTAuth) StoreVerify(ID int64, name string, accessToken string) bool {
	lock := svc.Mu.NewMutex(fmt.Sprintf("store-verify-lock-%d", ID))
	// 获取锁
	if err := lock.Lock(); err != nil {
		// 处理获取锁失败的情况
		return false
	}
	defer lock.Unlock()
	token := fmt.Sprintf("auth:access-token:%v:%s", ID, name)
	return svc.Client.Get(context.Background(), token).Val() == accessToken
}

// Blacklist 将令牌添加到 Redis 黑名单并设置过期时间。
func (svc *JWTAuth) Blacklist(accessToken string) error {
	lock := svc.Mu.NewMutex(fmt.Sprintf("blacklist-lock-%d", uuid.New()))
	// 获取锁
	if err := lock.Lock(); err != nil {
		return err
	}
	defer lock.Unlock()

	// 将令牌添加到 Redis，并设置过期时间
	return svc.Client.Set(context.Background(), accessToken, "blacklisted", expiry).Err()
}

// IsBlacklisted 检查令牌是否在黑名单中。
func (svc *JWTAuth) IsBlacklisted(accessToken string) (bool, error) {
	lock := svc.Mu.NewMutex(fmt.Sprintf("is-blacklisted-lock-%d", uuid.New()))
	// 获取锁
	if err := lock.Lock(); err != nil {
		// 处理获取锁失败的情况
		return false, err
	}
	defer lock.Unlock()
	// 检查令牌是否存在于 Redis 中
	_, err := svc.Client.Get(context.Background(), accessToken).Result()
	if err == goredislib.Nil {
		return false, nil
	} else if err != nil {
		return false, err
	}
	return true, nil
}

// CleanupExpiredTokens 定期清理 Redis 黑名单中的过期令牌。
func (svc *JWTAuth) CleanupExpiredTokens(interval time.Duration) {
	ticker := time.NewTicker(interval)
	defer ticker.Stop()

	for {
		select {
		case <-ticker.C:
			svc.cleanupExpiredTokens()
		}
	}
}

func (svc *JWTAuth) cleanupExpiredTokens() {
	lock := svc.Mu.NewMutex("cleanup-expired-tokens-lock")
	// 获取锁
	if err := lock.Lock(); err != nil {
		// 处理获取锁失败的情况
		return
	}
	defer lock.Unlock()

	// 遍历并从 Redis 中删除过期的键
	iter := svc.Client.Scan(context.Background(), 0, "*", 0).Iterator()
	for iter.Next(context.Background()) {
		key := iter.Val()
		ttl := svc.Client.TTL(context.Background(), key).Val()
		if ttl.Seconds() <= 0 {
			svc.Client.Del(context.Background(), key)
		}
	}
}

// handleSignals 在应用程序退出时处理信号
func handleSignals(client *goredislib.Client) {
	c := make(chan os.Signal, 1)
	signal.Notify(c, syscall.SIGINT, syscall.SIGTERM)
	go func() {
		<-c
		fmt.Println("Closing Redis connection...")
		if err := client.Close(); err != nil {
			fmt.Println("Error closing Redis connection:", err)
		} else {
			fmt.Println("Redis connection closed.")
		}
		os.Exit(0)
	}()
}
